# Authors: Rob Crittenden <rcritten@redhat.com>
#
# Copyright (C) 2013  Red Hat
# see file 'COPYING' for use and warranty information
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

import sys

# so we use our own copy of ipapython
from distutils.sysconfig import get_python_lib
sys.path.insert(0, "%s/%s" % (get_python_lib(), "ipaclient"))

import xmlrpclib
import socket
from krbtransport import KerbTransport
from kerberos import GSSError
from ipapython import config
import logging

class RPCClient:

    def __init__(self, verbose=False):
        self.verbose = verbose
        config.init_config()
        self.server = config.config.get_server()[0]

    def server_url(self, server):
        """Build the XML-RPC server URL from our configuration"""
        url = "https://" + server + "/ipa/xml"
        if self.verbose:
            logging.debug("Connecting to IPA server: %s" % url)
        return url

    def forward(self, name, *args, **kw):
        # FIXME: actually pass args and kw and add wrapping/unwrapping
        #        of args and kw and the response
        server = xmlrpclib.ServerProxy(self.server_url(self.server),
                                       KerbTransport(),
                                       verbose=self.verbose)

        command = getattr(server, name)
        response = command()
        return response


class IPAClient:

    def __init__(self,transport=None,verbose=False):
        if transport:
            self.local = True
            self.transport = transport
        else:
            self.local = False
            self.transport = RPCClient(verbose)

    def forward(self, command):
        try:
            result = self.transport.forward(command)['result']
        except GSSError, e:
            logging.error("Could not initialize GSSAPI: %s/%s" % (e[0][0], e[0][1]))
            return None
        except xmlrpclib.Fault, e:
            logging.error("XML-RPC error: %s" % e.faultString)
            return None
        except socket.error, e:
            logging.error("socket error: %s" % e)
            return None
        else:
            return result

if __name__ == '__main__':
    print IPAClient().forward('env')
